/*
	file automatically generated by make_test_files.pl
	Wed Nov  6 10:14:57 2002
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DISTR_CONT
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_distr_cont_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

#define COMPARE_SAMPLE_SIZE  (500)
#define VIOLATE_SAMPLE_SIZE   (20)

double pdf(double x, const UNUR_DISTR *distr);
double dpdf(double x, const UNUR_DISTR *distr);
double cdf(double x, const UNUR_DISTR *distr);





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_distr_cont_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_distr_cont_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(980736);
	unur_urng_fish_seed(980736);
	unur_urng_mstd_seed(980736);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(980736)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {980736,980736+1,980736+2,980736+3,980736+4,980736+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("distr_cont: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 35, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 35, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 38, (unur_distr_cont_set_pdf( distr, pdf )) );
n_tests_failed += check_errorcode( TESTLOG, 38, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 41, (unur_distr_cont_set_dpdf( distr, dpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 41, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 44, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 44, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 47, (unur_distr_cont_set_pdfstr( distr, "pdf" )) );
n_tests_failed += check_errorcode( TESTLOG, 47, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 50, (unur_distr_cont_set_cdfstr( distr, "cdf" )) );
n_tests_failed += check_errorcode( TESTLOG, 50, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 53, (unur_distr_cont_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 53, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 56, (unur_distr_cont_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 56, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 59, (unur_distr_cont_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 59, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 62, (unur_distr_cont_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 62, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
	distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 69, (unur_distr_cont_set_pdf( distr, pdf )) );
n_tests_failed += check_errorcode( TESTLOG, 69, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 72, (unur_distr_cont_set_dpdf( distr, dpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 72, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 75, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 75, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 78, (unur_distr_cont_set_pdfstr( distr, "exp(-x)" )) );
n_tests_failed += check_errorcode( TESTLOG, 78, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 81, (unur_distr_cont_set_cdfstr( distr, "exp(-x)" )) );
n_tests_failed += check_errorcode( TESTLOG, 81, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 84, (unur_distr_cont_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 84, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 87, (unur_distr_cont_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 87, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 90, (unur_distr_cont_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 90, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 93, (unur_distr_cont_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 93, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 100, (unur_distr_cont_set_pdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 100, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 103, (unur_distr_cont_set_dpdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 103, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 106, (unur_distr_cont_set_cdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 106, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 109, (unur_distr_cont_set_pdfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 109, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 112, (unur_distr_cont_set_cdfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 112, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 115, (unur_distr_cont_set_pdfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 115, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double pv[] = {1.,2.,3.};
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 123, (unur_distr_cont_set_pdfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 123, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 126, (unur_distr_cont_set_pdfstr( distr, "x * y * z" )) );
n_tests_failed += check_errorcode( TESTLOG, 126, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 129, (unur_distr_cont_set_pdfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 129, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 132, (unur_distr_cont_set_cdfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 132, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 135, (unur_distr_cont_set_cdfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 135, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 138, (unur_distr_cont_set_pdfparams( distr, pv, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 138, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 141, (unur_distr_cont_set_pdfparams( distr, pv, UNUR_DISTR_MAXPARAMS +1 )) );
n_tests_failed += check_errorcode( TESTLOG, 141, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 144, (unur_distr_cont_set_domain( distr, 0., -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 144, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 147, (unur_distr_cont_set_pdfarea( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 147, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   double a, b;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 160, ((void *) unur_distr_cont_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 160, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 163, ((void *) unur_distr_cont_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 163, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 166, ((void *) unur_distr_cont_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 166, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 169, (unur_distr_cont_get_pdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 169, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 172, (unur_distr_cont_get_dpdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 172, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 175, (unur_distr_cont_get_cdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 175, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 178, (unur_distr_cont_get_pdfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 178, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 181, (unur_distr_cont_get_domain( distr, &a, &b )) );
n_tests_failed += check_errorcode( TESTLOG, 181, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 184, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 184, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 187, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 187, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   double a,b;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 195, ((void *) unur_distr_cont_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 198, ((void *) unur_distr_cont_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 198, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 201, ((void *) unur_distr_cont_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 201, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 204, (unur_distr_cont_get_pdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 204, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 207, (unur_distr_cont_get_dpdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 207, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 210, (unur_distr_cont_get_cdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 210, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 213, (unur_distr_cont_get_pdfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 213, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 216, (unur_distr_cont_get_domain( distr, &a, &b )) );
n_tests_failed += check_errorcode( TESTLOG, 216, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 219, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 219, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 222, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 222, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* parameters not unknown */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 229, (unur_distr_cont_get_pdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 229, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 232, (unur_distr_cont_get_dpdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 232, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 235, (unur_distr_cont_get_cdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 235, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 238, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 238, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 241, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 241, UNUR_ERR_DISTR_GET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 252, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 252, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 255, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 255, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 262, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 262, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 265, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 265, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 272, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 272, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 275, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 275, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 294, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 294, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 297, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 297, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 300, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 300, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 307, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 307, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 310, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 310, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 313, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 313, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 320, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 320, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 323, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 323, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 326, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 326, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 0 */

/* free distributions */
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


double pdf(double x, const UNUR_DISTR *distr) { return 1.; }
double dpdf(double x, const UNUR_DISTR *distr) { return 1.; }
double cdf(double x, const UNUR_DISTR *distr) { return 1.; }



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_distr_cont_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_distr_cont_set_verify(UNUR_PAR *par, int verify) {return 0;}
